
local floor, ceil = math.floor, math.ceil
local min, max = math.min, math.max

local lg = love.graphics
lg.setColour, lg.setColor = lg.setColor, nil

math.randomseed(os.time())
love.window.setMode(600, 600, { resizable = true })
lg.setFont(lg.newFont(12))

local width, height = 4, 4
local grid = {}

local colours = {
	[ 1] = { 11/11, 11/11, 11/11 },
	[ 2] = { 10/11, 10/11, 10/11 },
	[ 3] = {  9/11,  9/11,  9/11 },
	[ 4] = {  8/11,  8/11,  8/11 },
	[ 5] = {  7/11,  7/11,  7/11 },
	[ 6] = {  6/11,  6/11,  6/11 },
	[ 7] = {  5/11,  5/11,  5/11 },
	[ 8] = {  4/11,  4/11,  4/11 },
	[ 9] = {  3/11,  3/11,  3/11 },
	[10] = {  2/11,  2/11,  2/11 },
	[11] = {  1/11,  1/11,  1/11 },
}
assert(2^11 == 2048)

local function addrandomly()
	local value = math.random(1,2)
	local index repeat
		index = math.random(0,width*height-1)
	until not grid[index]
	grid[index] = value
end

local state = "begin"

local function getr()
	local t = love.timer.getTime()
	local w, h = lg.getDimensions()
	local r = min(w,h)
	local pad = min(60, r*0.1)*(0.98 + 0.2 * math.sin(t/8*math.pi*2))
	local xshif = max(0,floor((w-h)/2))
	local yshif = max(0,floor((h-w)/2))
	return r-pad*2, pad, xshif, yshif
end

local function perr(x,y)
	local r, pad, xshif, yshif = getr()
	x = (x - pad - xshif) / r
	y = (y - pad - yshif) / r
	return x, y
end

local drawers = {
	begin = function()
		lg.setColour(0.80, 0.90, 0.80)
		lg.rectangle("fill", 0,0, 1,1)
	end,
	playing = function()
		local tp = love.timer.getTime()*math.pi*2
		lg.setColour(0.85, 0.85, 0.85+0.05*math.sin(tp/5))
		lg.rectangle("fill", 0,0, 1,1)
		for x = 0, width-1 do
			for y = 0, height-1 do
				local v = grid[x+y*width]
				if v then
					lg.setColour(colours[v])
					lg.rectangle("fill", x*0.25+0.01, y*0.25+0.01, 0.23, 0.23)
					lg.setColour(0,0,0)
					for i = 1, v do
						lg.circle("fill", x*0.25+0.01+0.04*i, y*0.25+0.01+0.04, 0.02)
					end
				end
			end
		end
	end,
	over = function()
		lg.setColour(0.90, 0.80, 0.80)
		lg.rectangle("fill", 0,0, 1,1)
	end,
}

local clickers = {
	begin = function(x,y)
		state = "playing"
		addrandomly()
		addrandomly()
	end,
	playing = function(x,y)
	end,
	over = function(x,y)
		state = "begin"
	end,
}

local keybers = {
	begin = function()
	end,
	playing = function(key)
		
		-- I am ashamed
		
		if key == "left" then
			for y = 0, 3 do
				for x = 1, 3 do
					if grid[x+y*width] then
						for xx = x-1, 0, -1 do
							if not grid[xx+y*width] then
								grid[xx+y*width] = grid[(xx+1)+y*width]
								grid[(xx+1)+y*width] = nil
							elseif grid[xx+y*width] == grid[(xx+1)+y*width] then
								grid[xx+y*width] = grid[xx+y*width] + 1
								grid[(xx+1)+y*width] = nil
								break
							else
								break
							end
						end
					end
				end
			end
		elseif key == "right" then
			for y = 0, 3 do
				for x = 2, 0, -1 do
					if grid[x+y*width] then
						for xx = x+1, 3 do
							if not grid[xx+y*width] then
								grid[xx+y*width] = grid[(xx-1)+y*width]
								grid[(xx-1)+y*width] = nil
							elseif grid[xx+y*width] == grid[(xx-1)+y*width] then
								grid[xx+y*width] = grid[xx+y*width] + 1
								grid[(xx-1)+y*width] = nil
								break
							else
								break
							end
						end
					end
				end
			end
		elseif key == "up" then
			for x = 0, 3 do
				for y = 1, 3 do
					if grid[x+y*width] then
						for yy = y-1, 0, -1 do
							if not grid[x+yy*width] then
								grid[x+yy*width] = grid[x+(yy+1)*width]
								grid[x+(yy+1)*width] = nil
							elseif grid[x+yy*width] == grid[x+(yy+1)*width] then
								grid[x+yy*width] = grid[x+yy*width] + 1
								grid[x+(yy+1)*width] = nil
								break
							else
								break
							end
						end
					end
				end
			end
		elseif key == "down" then
			for x = 0, 3 do
				for y = 2, 0, -1 do
					if grid[x+y*width] then
						for yy = y+1, 3 do
							if not grid[x+yy*width] then
								grid[x+yy*width] = grid[x+(yy-1)*width]
								grid[x+(yy-1)*width] = nil
							elseif grid[x+yy*width] == grid[x+(yy-1)*width] then
								grid[x+yy*width] = grid[x+yy*width] + 1
								grid[x+(yy-1)*width] = nil
								break
							else
								break
							end
						end
					end
				end
			end
		end

		addrandomly()

		local filled = 0
		for i = 0, width*height-1 do
			if grid[i] == 11 then
				state = over
			elseif grid[i] then
				filled = filled + 1
			end
		end
		if filled == width*height then
			state = over
		end
	end,
	over = function()
	end,
}

function love.draw()
	lg.clear(1,1,1)
	do
		local r, pad, xshif, yshif = getr()
		lg.translate(pad, pad)
		lg.translate(xshif, yshif)
		lg.scale(r)
	end

	drawers[state]()
end

function love.mousepressed(x,y)
	clickers[state](perr(x,y))
end

function love.keypressed(key, scancode)
	keybers[state](key, scancode)
end
